<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 14/11/2014
 * Time: 16:42
 */

namespace Folha\Logic\ValesBeneficios\IntegracaoEletronicaVT;

use \Folha\Entities\VabIntegtransph;

use \Folha\Entities\VabIntegtranspm;

use \HCM\Util\DateUtil;

use \Geral\Util\NovoCodigo;

use HCM\Util\FPA;

class Controlador {

    const OTIMO      = 10;
    const TRANSFACIL = 11;

    private $tipoIntegracao;


    /**
     * Insere as informações da integração no mestre.
     * @param array $infoIntegracao
     * @param int $nrorg
     * @param \DateTime $dtmescompetencia
     * @param \DateTime $dtIniIntegracao
     * @param \DateTime $dtFimIntegracao
     * @throws \Exception
     */
    private function insereIntegMestre( $infoIntegracao, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao)
    {
        try {
            $codigo = NovoCodigo::geraCodigo($nrorg, 'VAB_INTEGTRANSPM', 1, 21, true, true);
            $registro = new VabIntegtranspm();

            $registro->setNrintegtranspm($codigo);
            $registro->setDtinivalidadeinteg($dtIniIntegracao);
            $registro->setDtfinvalidadeinteg($dtFimIntegracao);
            $registro->setIdativo('S');
            $registro->setDtinclusao(DateUtil::getDataAtual());
            $registro->setDtultatu(DateUtil::getDataAtual());
            $registro->setCdoperinclusao(FPA::getCdoperador());
            $registro->setCdoperultatu(FPA::getCdOperador());
            $registro->setNrorg(FPA::getNrorgTrabalho());
            $registro->setNrorginclusao(FPA::getNrorgTrabalho());
            $registro->setNrorgultatu(FPA::getNrorgTrabalho());

            $resultado = \Folha\Repositories\VabIntegtranspm::inserirNovo($registro);

            if($resultado) {
                $this->insereIntegHistor($codigo, $infoIntegracao, $nrorg, $dtmescompetencia, 'N');
            }

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Insere as informações da integração no histórico.
     * @param int $nrMestre
     * @param array $infoIntegracao
     * @param int $nrorg
     * @param \DateTime $dtmescompetencia
     * @param string $reserva
     * @throws \Exception
     */
    private function insereIntegHistor( $nrMestre, $infoIntegracao, $nrorg, \DateTime $dtmescompetencia, $reserva = 'N', $vrMultiplo = 0.01, $nrTipoIntegracao = 1, $nrmatricula = null, $cdformatointeg = null, $nrestruturam = null )
    {
        try {
            $codigo = NovoCodigo::geraCodigo($nrorg, 'VAB_INTEGTRANSPH', 1, 21, true, true);

            $registro = new VabIntegtransph();

            $registro->setNrintegtransph($codigo);
            $registro->setNrintegtranspm($nrMestre);
            $registro->setCdintegtransp($infoIntegracao['NUMERO_CARTAO']);
            $registro->setVrsaldoultconsulta($infoIntegracao['SALDO_CARTAO']);
            $registro->setDtmescompinteg($dtmescompetencia);
            $registro->setIdintegreserva($reserva);
            $registro->setIdativo('S');
            $registro->setNrtipolinhavinc($this->tipoIntegracao);
            $registro->setVrmultiplointeg($vrMultiplo);
            $registro->setDtultutilizacao(DateUtil::subtraiIntervalo(DateUtil::getDataAtual(),2));
            $registro->setNrtipointegtransp($nrTipoIntegracao);
            $registro->setDtultatu(DateUtil::getDataAtual());
            $registro->setDtinclusao(DateUtil::getDataAtual());
            $registro->setCdoperinclusao(FPA::getCdOperador());
            $registro->setCdoperultatu(FPA::getCdOperador());
            $registro->setNrorg(FPA::getNrorgTrabalho());
            $registro->setNrorginclusao(FPA::getNrorgTrabalho());
            $registro->setNrorgultatu(FPA::getNrorgTrabalho());
            /*Rafael Pickbrenner PCK 31/07/2015 inclusão de get's e set's */
            $registro->setNrmatricula($nrmatricula);
            $registro->setCdformatointeg($cdformatointeg);
            $registro->setNrestruturam($nrestruturam);

            \Folha\Repositories\VabIntegtransph::inserirNovo($registro);

        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Altera as informações da integração no histórico.
     * @param VabIntegtransph $registro
     * @param array $infoIntegracao
     * @throws \Exception
     */
    private function alteraIntegHistor( VabIntegtransph $registro, $infoIntegracao)
    {
        try {

            $registro->setVrsaldoultconsulta($infoIntegracao['SALDO_CARTAO']);
            $registro->setDtultatu(DateUtil::getDataAtual());
            $registro->setCdoperultatu(FPA::getCdOperador());
            $registro->setNrorgultatu(FPA::getNrorgTrabalho());
            $registro->setDtultutilizacao(DateUtil::subtraiIntervalo(DateUtil::getDataAtual(),2));

            \Folha\Repositories\VabIntegtransph::atualizarRegistro($registro);

        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Inicia a integração e persiste os dados.
     * @param int $nrorg
     * @param \DateTime $dtmescompetencia
     * @param String $caminhoArquivo
     * @param \DateTime $dtIniIntegracao
     * @param \DateTime $dtFimIntegracao
     * @throws \Exception
     */
    public function executaIntegracao( $nrorg, $dtmescompetencia, $caminhoArquivo, $dtIniIntegracao, $dtFimIntegracao)
    {
        try {
            switch ($this->tipoIntegracao) {
                case(self::OTIMO): {
                    $integracao = new LeitorOtimoII();
                    break;
                }
                case(self::TRANSFACIL): {
                    $integracao = new LeitorTransfacil();
                    break;
                }
                default: {
                $integracao = '';
                }
            }

            $infos = $integracao->getInfo($caminhoArquivo);
            foreach($infos as $info){
                $this->insereIntegração($info, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao);
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Construtor da classe
     * @param int $tipoIntegracao
     */
    public function __construct( $tipoIntegracao)
    {
        $this->tipoIntegracao = $tipoIntegracao;
    }

    /**
     * Buscar registro do cartão
     *
     * @param int $nrorg
     * @param String $codIntegracao
     * @param \DateTime $dtmescompetencia
     *
     * @return VabIntegtransph
     * @throws \Exception
     */
    private function buscaIntegracao( $nrorg , $codIntegracao, $dtmescompetencia)
    {
        try {
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDINTEGTRANSP' => trim($codIntegracao),
                'P_DTMESCOMPETENCIA' => $dtmescompetencia->format('d/m/Y'),
            );
            $where = "NRORG = :P_NRORG AND NRINTEGTRANSPH IN (SELECT IT.NRINTEGTRANSPH FROM VAB_INTEGTRANSPH IT WHERE IT.NRORG = :P_NRORG AND :P_CDINTEGTRANSP LIKE IT.CDINTEGTRANSP || '%' AND IT.DTMESCOMPINTEG = ( SELECT MAX(M.DTMESCOMPINTEG) FROM VAB_INTEGTRANSPH M WHERE M.CDINTEGTRANSP = IT.CDINTEGTRANSP AND M.NRORG = IT.NRORG AND M.DTMESCOMPINTEG <= :P_DTMESCOMPETENCIA ) )";
            $resultado = VabIntegtransph::getObjectByWhere($where, $params, 'DTMESCOMPINTEG DESC');

            if ($resultado) {
                return $resultado;
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Realiza a inserção dos valores da importação
     * @param array $info
     * @param int $nrorg
     * @param \DateTime $dtmescompetencia
     * @param \DateTime $dtIniIntegracao
     * @param \DateTime $dtFimIntegracao
     * @throws \Exception
     */
    private function insereIntegração($info, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao)
    {
        $ultHistorico = $this->buscaIntegracao($nrorg, $info['NUMERO_CARTAO'], $dtmescompetencia);
        if (!$ultHistorico) {
            $this->insereIntegMestre($info, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao);
        } elseif($dtmescompetencia->format('d/m/Y') == $ultHistorico->getDtmescompinteg()->format('d/m/Y')) {
            $this->alteraIntegHistor($ultHistorico, $info);
        }else{
            $this->insereIntegHistor($ultHistorico->getNrintegtranspm(), $info, $nrorg, $dtmescompetencia, $ultHistorico->getIdintegreserva(), $ultHistorico->getVrmultiplointeg(), $ultHistorico->getNrtipointegtransp(), $ultHistorico->getNrmatricula(), $ultHistorico->getCdformatointeg(), $ultHistorico->getNrestruturam());
        }
    }
}